@@ -35,6 +35,7 @@
 struct _EAddressbookModelPrivate {
 	EClientCache *client_cache;
 	gulong client_notify_readonly_handler_id;
+	gulong client_notify_capabilities_handler_id;
 
 	EBookClient *book_client;
 	gchar *query_str;
@@ -68,6 +69,7 @@ enum {
 enum {
 	WRITABLE_STATUS,
 	STATUS_MESSAGE,
+	BEFORE_SEARCH,
 	SEARCH_STARTED,
 	SEARCH_RESULT,
 	FOLDER_BAR_MESSAGE,
@@ -347,6 +349,25 @@ addressbook_model_client_notify_readonly_cb (EClientCache *client_cache,
 	}
 }
 
+static gboolean addressbook_model_idle_cb (EAddressbookModel *model);
+
+static void
+addressbook_model_client_notify_capabilities_cb (EClientCache *client_cache,
+						 EClient *client,
+						 GParamSpec *pspec,
+						 EAddressbookModel *model)
+{
+	if (!E_IS_BOOK_CLIENT (client))
+		return;
+
+	if (E_BOOK_CLIENT (client) == model->priv->book_client &&
+	    model->priv->client_view_idle_id == 0) {
+		model->priv->client_view_idle_id = g_idle_add (
+			(GSourceFunc) addressbook_model_idle_cb,
+			g_object_ref (model));
+	}
+}
+
 static void
 client_view_ready_cb (GObject *source_object,
                       GAsyncResult *result,
@@ -372,6 +393,8 @@ client_view_ready_cb (GObject *source_object,
 		return;
 	}
 
+	g_signal_emit (model, signals[BEFORE_SEARCH], 0);
+
 	remove_book_view (model);
 	free_data (model);
 
@@ -557,6 +580,13 @@ addressbook_model_dispose (GObject *object)
 		model->priv->client_notify_readonly_handler_id = 0;
 	}
 
+	if (model->priv->client_notify_capabilities_handler_id > 0) {
+		g_signal_handler_disconnect (
+			model->priv->client_cache,
+			model->priv->client_notify_capabilities_handler_id);
+		model->priv->client_notify_capabilities_handler_id = 0;
+	}
+
 	g_clear_object (&model->priv->client_cache);
 	g_clear_object (&model->priv->book_client);
 
@@ -598,6 +628,12 @@ addressbook_model_constructed (GObject *object)
 		G_CALLBACK (addressbook_model_client_notify_readonly_cb),
 		model);
 	model->priv->client_notify_readonly_handler_id = handler_id;
+
+	handler_id = g_signal_connect (
+		client_cache, "client-notify::capabilities",
+		G_CALLBACK (addressbook_model_client_notify_capabilities_cb),
+		model);
+	model->priv->client_notify_capabilities_handler_id = handler_id;
 }
 
 static void
@@ -681,6 +717,15 @@ e_addressbook_model_class_init (EAddressbookModelClass *class)
 		G_TYPE_STRING,
 		G_TYPE_INT);
 
+	signals[BEFORE_SEARCH] = g_signal_new (
+		"before-search",
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		/* G_STRUCT_OFFSET (EAddressbookModelClass, before_search) */ 0,
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
+
 	signals[SEARCH_STARTED] = g_signal_new (
 		"search_started",
 		G_OBJECT_CLASS_TYPE (object_class),
@@ -851,6 +896,7 @@ e_addressbook_model_contact_at (EAddressbookModel *model,
                                 gint index)
 {
 	g_return_val_if_fail (E_IS_ADDRESSBOOK_MODEL (model), NULL);
+	g_return_val_if_fail (index >= 0 && (guint) index < model->priv->contacts->len, NULL);
 
 	return model->priv->contacts->pdata[index];
 }
@@ -862,11 +908,6 @@ e_addressbook_model_find (EAddressbookModel *model,
 	GPtrArray *array;
 	gint ii;
 
-	/* XXX This searches for a particular EContact instance,
-	 *     as opposed to an equivalent but possibly different
-	 *     EContact instance.  Might have to revise this in
-	 *     the future. */
-
 	g_return_val_if_fail (E_IS_ADDRESSBOOK_MODEL (model), -1);
 	g_return_val_if_fail (E_IS_CONTACT (contact), -1);
 
@@ -874,7 +915,8 @@ e_addressbook_model_find (EAddressbookModel *model,
 	for (ii = 0; ii < array->len; ii++) {
 		EContact *candidate = array->pdata[ii];
 
-		if (contact == candidate)
+		if (contact == candidate ||
+		    g_strcmp0 (e_contact_get_const (contact, E_CONTACT_UID), e_contact_get_const (candidate, E_CONTACT_UID)) == 0)
 			return ii;
 	}
 
