@@ -51,7 +51,7 @@ ECompEditorPropertyPart *
 ECompEditorPropertyPart *
 		e_comp_editor_property_part_classification_new	(void);
 ECompEditorPropertyPart *
-		e_comp_editor_property_part_status_new		(void);
+		e_comp_editor_property_part_status_new		(ICalComponentKind kind);
 ECompEditorPropertyPart *
 		e_comp_editor_property_part_priority_new	(void);
 ECompEditorPropertyPart *
@@ -62,6 +62,9 @@ ECompEditorPropertyPart *
 		e_comp_editor_property_part_transparency_new	(void);
 ECompEditorPropertyPart *
 		e_comp_editor_property_part_color_new		(void);
+ECompEditorPropertyPart *
+		e_comp_editor_property_part_estimated_duration_new
+								(void);
 
 G_END_DECLS
 
