@@ -42,6 +42,7 @@
 #include "e-cal-ops.h"
 #include "e-calendar-view.h"
 #include "e-cell-date-edit-text.h"
+#include "e-cell-estimated-duration.h"
 #include "itip-utils.h"
 #include "print.h"
 #include "misc.h"
@@ -330,15 +331,8 @@ task_table_dispose (GObject *object)
 		priv->model = NULL;
 	}
 
-	if (priv->copy_target_list != NULL) {
-		gtk_target_list_unref (priv->copy_target_list);
-		priv->copy_target_list = NULL;
-	}
-
-	if (priv->paste_target_list != NULL) {
-		gtk_target_list_unref (priv->paste_target_list);
-		priv->paste_target_list = NULL;
-	}
+	g_clear_pointer (&priv->copy_target_list, gtk_target_list_unref);
+	g_clear_pointer (&priv->paste_target_list, gtk_target_list_unref);
 
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_task_table_parent_class)->dispose (object);
@@ -423,6 +417,16 @@ task_table_constructed (GObject *object)
 		E_CELL_DATE_EDIT (popup_cell),
 		e_task_table_get_current_time, task_table, NULL);
 
+	cell = e_cell_estimated_duration_new (NULL, GTK_JUSTIFY_LEFT);
+	g_object_set (
+		cell,
+		"strikeout_column", E_CAL_MODEL_TASKS_FIELD_STRIKEOUT,
+		"bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
+		"bg_color_column", E_CAL_MODEL_FIELD_COLOR,
+		NULL);
+	e_table_extras_add_cell (extras, "estimatedduration", cell);
+	g_object_unref (cell);
+
 	/*
 	 * Combo fields.
 	 */
@@ -490,6 +494,7 @@ task_table_constructed (GObject *object)
 		NULL);
 
 	popup_cell = e_cell_combo_new ();
+	e_cell_combo_use_tabular_numbers (E_CELL_COMBO (popup_cell));
 	e_cell_popup_set_child (E_CELL_POPUP (popup_cell), cell);
 	g_object_unref (cell);
 
@@ -885,7 +890,7 @@ task_table_query_tooltip (GtkWidget *widget,
 			g_string_append (tmp2, e_cal_component_text_get_value (text));
 			if (len > 1024) {
 				g_string_set_size (tmp2, 1023);
-				g_string_append (tmp2, "…");
+				g_string_append (tmp2, _("…"));
 				break;
 			}
 		}
