@@ -198,8 +198,10 @@ etdp_itt_to_zone (ICalTime *itt,
 		zone = i_cal_timezone_get_utc_timezone ();
 	}
 
-	if (zone)
+	if (zone) {
 		i_cal_time_convert_timezone (itt, zone, default_zone);
+		i_cal_time_set_timezone (itt, default_zone);
+	}
 }
 
 static gchar *
@@ -2818,6 +2820,8 @@ e_to_do_pane_class_init (EToDoPaneClass *klass)
 
 	g_type_class_add_private (klass, sizeof (EToDoPanePrivate));
 
+	gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (klass), "EToDoPane");
+
 	object_class = G_OBJECT_CLASS (klass);
 	object_class->set_property = e_to_do_pane_set_property;
 	object_class->get_property = e_to_do_pane_get_property;
