@@ -162,6 +162,30 @@ action_print_preview_cb (GtkAction *action,
 	e_msg_composer_print (composer, print_action);
 }
 
+static void
+action_save_ready_cb (GObject *source_object,
+		      GAsyncResult *result,
+		      gpointer user_data)
+{
+	EMsgComposer *composer = user_data;
+	GError *error = NULL;
+
+	g_return_if_fail (E_IS_MSG_COMPOSER (composer));
+	g_return_if_fail (E_IS_HTML_EDITOR (source_object));
+
+	if (!e_html_editor_save_finish (E_HTML_EDITOR (source_object), result, &error)) {
+		e_alert_submit (
+			E_ALERT_SINK (composer),
+			E_ALERT_NO_SAVE_FILE,
+			e_html_editor_get_filename (E_HTML_EDITOR (source_object)), error ? error->message : _("Unknown error"), NULL);
+	} else {
+		composer_set_content_editor_changed (composer);
+	}
+
+	g_object_unref (composer);
+	g_clear_error (&error);
+}
+
 static void
 action_save_cb (GtkAction *action,
                 EMsgComposer *composer)
@@ -169,7 +193,6 @@ action_save_cb (GtkAction *action,
 	EHTMLEditor *editor;
 	const gchar *filename;
 	gint fd;
-	GError *error = NULL;
 
 	editor = e_msg_composer_get_editor (composer);
 	filename = e_html_editor_get_filename (editor);
@@ -202,16 +225,7 @@ action_save_cb (GtkAction *action,
 	} else
 		close (fd);
 
-	if (!e_html_editor_save (editor, filename, TRUE, &error)) {
-		e_alert_submit (
-			E_ALERT_SINK (composer),
-			E_ALERT_NO_SAVE_FILE,
-			filename, error->message, NULL);
-		g_error_free (error);
-		return;
-	}
-
-	composer_set_content_editor_changed (composer);
+	e_html_editor_save (editor, filename, TRUE, NULL, action_save_ready_cb, g_object_ref (composer));
 }
 
 static void
@@ -544,6 +558,35 @@ static GtkToggleActionEntry toggle_entries[] = {
 	  FALSE }
 };
 
+static gboolean
+eca_transform_mode_html_to_boolean_cb (GBinding *binding,
+				       const GValue *source_value,
+				       GValue *target_value,
+				       gpointer not_used)
+{
+	g_value_set_boolean (target_value, g_value_get_enum (source_value) == E_CONTENT_EDITOR_MODE_HTML);
+
+	return TRUE;
+}
+
+static gboolean
+eca_mode_to_bool_hide_in_markdown_cb (GBinding *binding,
+				      const GValue *from_value,
+				      GValue *to_value,
+				      gpointer user_data)
+{
+	EContentEditorMode mode;
+
+	mode = g_value_get_enum (from_value);
+
+	g_value_set_boolean (to_value,
+		mode != E_CONTENT_EDITOR_MODE_MARKDOWN &&
+		mode != E_CONTENT_EDITOR_MODE_MARKDOWN_PLAIN_TEXT &&
+		mode != E_CONTENT_EDITOR_MODE_MARKDOWN_HTML);
+
+	return TRUE;
+}
+
 void
 e_composer_actions_init (EMsgComposer *composer)
 {
@@ -662,10 +705,12 @@ e_composer_actions_init (EMsgComposer *composer)
 		g_object_unref (gcr_gnupg_icon);
 	}
 
-	e_binding_bind_property (
-		cnt_editor, "html-mode",
+	e_binding_bind_property_full (
+		editor, "mode",
 		ACTION (PICTURE_GALLERY), "sensitive",
-		G_BINDING_SYNC_CREATE);
+		G_BINDING_SYNC_CREATE,
+		eca_transform_mode_html_to_boolean_cb,
+		NULL, NULL, NULL);
 
 	e_binding_bind_property (
 		cnt_editor, "editable",
@@ -697,6 +742,13 @@ e_composer_actions_init (EMsgComposer *composer)
 		e_html_editor_get_action (editor, "visually-wrap-long-lines"), "active",
 		G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
+	e_binding_bind_property_full (
+		editor, "mode",
+		e_html_editor_get_action (editor, "visually-wrap-long-lines"), "visible",
+		G_BINDING_SYNC_CREATE,
+		eca_mode_to_bool_hide_in_markdown_cb,
+		NULL, NULL, NULL);
+
 #if defined (ENABLE_SMIME)
 	visible = TRUE;
 #else
