@@ -39,24 +39,46 @@ GType e_scrolled_window_get_type (void) G_GNUC_CONST;
 G_DEFINE_TYPE (EScrolledWindow, e_scrolled_window, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
-e_scrolled_window_get_preferred_height (GtkWidget *widget,
-					gint *minimum_size,
-					gint *natural_size)
+e_scrolled_window_get_preferred_height_for_width (GtkWidget *widget,
+						  gint width,
+						  gint *minimum_size,
+						  gint *natural_size)
 {
 	GtkScrolledWindow *scrolled_window = GTK_SCROLLED_WINDOW (widget);
+	GtkWidget *child;
 	gint min_height, max_height;
 
 	/* Chain up to parent's method. */
-	GTK_WIDGET_CLASS (e_scrolled_window_parent_class)->get_preferred_height (widget, minimum_size, natural_size);
+	GTK_WIDGET_CLASS (e_scrolled_window_parent_class)->get_preferred_height_for_width (widget, width, minimum_size, natural_size);
 
 	min_height = gtk_scrolled_window_get_min_content_height (scrolled_window);
 	max_height = gtk_scrolled_window_get_max_content_height (scrolled_window);
 
 	if (min_height > 0 && min_height < *minimum_size)
-		*minimum_size = min_height + 1;
+		*minimum_size = min_height + 2;
 
 	if (max_height > 0 && max_height < *natural_size)
-		*natural_size = max_height + 1;
+		*natural_size = max_height + 2;
+
+	child = gtk_bin_get_child (GTK_BIN (widget));
+
+	if (child && width > 1) {
+		gint child_min_height = -1, child_natural_height = -1;
+
+		gtk_widget_get_preferred_height_for_width (child, width, &child_min_height, &child_natural_height);
+
+		if (*minimum_size > child_min_height && child_min_height > 0)
+			*minimum_size = child_min_height + 2;
+
+		if (*natural_size > child_natural_height && child_natural_height > 0)
+			*natural_size = child_natural_height + 2;
+	}
+}
+
+static GtkSizeRequestMode
+e_scrolled_window_get_request_mode (GtkWidget *widget)
+{
+	return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
 }
 
 static void
@@ -65,7 +87,8 @@ e_scrolled_window_class_init (EScrolledWindowClass *class)
 	GtkWidgetClass *widget_class;
 
 	widget_class = GTK_WIDGET_CLASS (class);
-	widget_class->get_preferred_height = e_scrolled_window_get_preferred_height;
+	widget_class->get_preferred_height_for_width = e_scrolled_window_get_preferred_height_for_width;
+	widget_class->get_request_mode = e_scrolled_window_get_request_mode;
 }
 
 static void
@@ -161,6 +184,8 @@ alert_bar_show_alert (EAlertBar *alert_bar)
 		gtk_activatable_set_related_action (GTK_ACTIVATABLE (widget), action);
 		gtk_box_pack_end (GTK_BOX (action_area), widget, FALSE, FALSE, 0);
 
+		e_alert_update_destructive_action_style (action, widget);
+
 		link = g_list_next (link);
 	}
 
@@ -350,6 +375,29 @@ alert_bar_dispose (GObject *object)
 	G_OBJECT_CLASS (e_alert_bar_parent_class)->dispose (object);
 }
 
+static void
+alert_bar_add_css_style (GtkWidget *widget,
+			 const gchar *css)
+{
+	GtkCssProvider *provider;
+	GError *error = NULL;
+
+	provider = gtk_css_provider_new ();
+
+	if (gtk_css_provider_load_from_data (provider, css, -1, &error)) {
+		GtkStyleContext *style_context;
+
+		style_context = gtk_widget_get_style_context (widget);
+
+		gtk_style_context_add_provider (style_context, GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	} else {
+		g_warning ("%s: Failed to parse CSS for %s: %s", G_STRFUNC, G_OBJECT_TYPE_NAME (widget), error ? error->message : "Unknown error");
+	}
+
+	g_clear_object (&provider);
+	g_clear_error (&error);
+}
+
 static void
 alert_bar_constructed (GObject *object)
 {
@@ -412,6 +460,16 @@ alert_bar_constructed (GObject *object)
 	g_signal_connect (priv->message_label, "size-allocate",
 		G_CALLBACK (alert_bar_message_label_size_allocate_cb), object);
 
+	widget = gtk_bin_get_child (GTK_BIN (container));
+
+	if (GTK_IS_VIEWPORT (widget)) {
+		gtk_viewport_set_shadow_type (GTK_VIEWPORT (widget), GTK_SHADOW_NONE);
+
+		alert_bar_add_css_style (widget, "viewport { background: none; border: none; }");
+	}
+
+	alert_bar_add_css_style (container, "scrolledwindow { background: none; border: none; }");
+
 	/* Disable animation of the revealer, until GtkInfoBar's bug #710888 is fixed */
 	revealer = gtk_widget_get_template_child (GTK_WIDGET (object), GTK_TYPE_INFO_BAR, "revealer");
 	if (revealer) {
