@@ -914,7 +914,7 @@ e_attachment_store_run_save_dialog (EAttachmentStore *store,
 			name = _("attachment.dat");
 
 		allocated = g_strdup (name);
-		e_filename_make_safe (allocated);
+		e_util_make_safe_filename (allocated);
 
 		gtk_file_chooser_set_current_name (file_chooser, allocated);
 
@@ -1523,26 +1523,11 @@ attachment_store_save_context_free (SaveContext *save_context)
 	/* So should the error. */
 	g_warn_if_fail (save_context->error == NULL);
 
-	if (save_context->destination) {
-		g_object_unref (save_context->destination);
-		save_context->destination = NULL;
-	}
-
-	g_free (save_context->filename_prefix);
-	save_context->filename_prefix = NULL;
-
-	if (save_context->fresh_directory) {
-		g_object_unref (save_context->fresh_directory);
-		save_context->fresh_directory = NULL;
-	}
-
-	if (save_context->trash_directory) {
-		g_object_unref (save_context->trash_directory);
-		save_context->trash_directory = NULL;
-	}
-
+	g_clear_object (&save_context->destination);
+	g_clear_object (&save_context->fresh_directory);
+	g_clear_object (&save_context->trash_directory);
+	g_clear_pointer (&save_context->filename_prefix, g_free);
 	g_strfreev (save_context->uris);
-
 	g_slice_free (SaveContext, save_context);
 }
 
@@ -1575,7 +1560,7 @@ attachment_store_move_file (SaveContext *save_context,
 	 * we get a spurious G_IO_ERROR_WOULD_MERGE error and the
 	 * user has to try saving attachments again. */
 	tmpl = g_strdup_printf (PACKAGE "-%s-XXXXXX", g_get_user_name ());
-	path = e_mktemp (tmpl);
+	path = e_mkdtemp (tmpl);
 	g_free (tmpl);
 
 	save_context->trash_directory = g_file_new_for_path (path);
