@@ -109,7 +109,7 @@ part_combobox_changed (GtkComboBox *combobox,
 
 	g_return_if_fail (i == index);
 
-	/* dont update if we haven't changed */
+	/* don't update if we haven't changed */
 	if (!strcmp (part->title, data->part->title))
 		return;
 
@@ -811,9 +811,16 @@ filter_rule_xml_decode (EFilterRule *rule,
 }
 
 static void
-filter_rule_build_code (EFilterRule *rule,
-                        GString *out)
+filter_rule_build_code_for_parts (EFilterRule *rule,
+				  GList *parts,
+				  gboolean without_match_all,
+				  gboolean force_match_all,
+				  GString *out)
 {
+	g_return_if_fail (rule != NULL);
+	g_return_if_fail (parts != NULL);
+	g_return_if_fail (out != NULL);
+
 	switch (rule->threading) {
 	case E_FILTER_THREAD_NONE:
 		break;
@@ -831,25 +838,97 @@ filter_rule_build_code (EFilterRule *rule,
 		break;
 	}
 
-	if (rule->threading != E_FILTER_THREAD_NONE)
+	if ((rule->threading != E_FILTER_THREAD_NONE && !without_match_all) || force_match_all)
 		g_string_append (out, "(match-all ");
 
-	switch (rule->grouping) {
-	case E_FILTER_GROUP_ALL:
-		g_string_append (out, " (and\n  ");
-		break;
-	case E_FILTER_GROUP_ANY:
-		g_string_append (out, " (or\n  ");
-		break;
-	default:
-		g_warning ("Invalid grouping");
+	if (parts->next) {
+		switch (rule->grouping) {
+		case E_FILTER_GROUP_ALL:
+			g_string_append (out, " (and\n  ");
+			break;
+		case E_FILTER_GROUP_ANY:
+			g_string_append (out, " (or\n  ");
+			break;
+		default:
+			g_warning ("Invalid grouping");
+		}
 	}
 
-	e_filter_part_build_code_list (rule->parts, out);
-	g_string_append (out, ")\n");
+	e_filter_part_build_code_list (parts, out);
 
-	if (rule->threading != E_FILTER_THREAD_NONE)
-		g_string_append (out, "))\n");
+	if (parts->next)
+		g_string_append (out, ")\n");
+
+	if (rule->threading != E_FILTER_THREAD_NONE) {
+		if (without_match_all && !force_match_all)
+			g_string_append (out, ")\n");
+		else
+			g_string_append (out, "))\n");
+	} else if (force_match_all) {
+		g_string_append (out, ")\n");
+	}
+}
+
+static void
+filter_rule_build_code (EFilterRule *rule,
+                        GString *out)
+{
+	GList *link;
+	gboolean has_body_search = FALSE;
+
+	if (!rule->parts)
+		return;
+
+	for (link = rule->parts; link && !has_body_search; link = g_list_next (link)) {
+		EFilterPart *part = link->data;
+
+		has_body_search = g_strcmp0 (part->name, "body") == 0;
+	}
+
+	if (has_body_search) {
+		GList *body_searches = NULL, *other_searches = NULL;
+
+		for (link = rule->parts; link; link = g_list_next (link)) {
+			EFilterPart *part = link->data;
+
+			if (g_strcmp0 (part->name, "body") == 0) {
+				body_searches = g_list_prepend (body_searches, part);
+			} else {
+				other_searches = g_list_prepend (other_searches, part);
+			}
+		}
+
+		if (other_searches && body_searches) {
+			switch (rule->grouping) {
+			case E_FILTER_GROUP_ALL:
+				g_string_append (out, "(and ");
+				break;
+			case E_FILTER_GROUP_ANY:
+				g_string_append (out, "(or ");
+				break;
+			default:
+				g_warning ("Invalid grouping");
+			}
+
+			body_searches = g_list_reverse (body_searches);
+			other_searches = g_list_reverse (other_searches);
+
+			filter_rule_build_code_for_parts (rule, other_searches, FALSE, TRUE, out);
+
+			g_string_append_c (out, ' ');
+
+			filter_rule_build_code_for_parts (rule, body_searches, TRUE, FALSE, out);
+
+			g_string_append_c (out, ')');
+		} else {
+			filter_rule_build_code_for_parts (rule, rule->parts, FALSE, FALSE, out);
+		}
+
+		g_list_free (body_searches);
+		g_list_free (other_searches);
+	} else {
+		filter_rule_build_code_for_parts (rule, rule->parts, FALSE, FALSE, out);
+	}
 }
 
 static void
@@ -895,59 +974,6 @@ ensure_scrolled_width_cb (GtkAdjustment *adj,
 		gtk_adjustment_get_upper (adj));
 }
 
-static void
-ensure_scrolled_height_cb (GtkAdjustment *adj,
-                           GParamSpec *param_spec,
-                           GtkScrolledWindow *scrolled_window)
-{
-	GtkWidget *toplevel;
-	GdkScreen *screen;
-	gint toplevel_height, scw_height, require_scw_height = 0, max_height;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (scrolled_window));
-	if (!toplevel || !gtk_widget_is_toplevel (toplevel))
-		return;
-
-	scw_height = gtk_widget_get_allocated_height (GTK_WIDGET (scrolled_window));
-
-	gtk_widget_get_preferred_height_for_width (gtk_bin_get_child (GTK_BIN (scrolled_window)),
-		gtk_widget_get_allocated_width (GTK_WIDGET (scrolled_window)),
-		&require_scw_height, NULL);
-
-	if (scw_height >= require_scw_height) {
-		if (require_scw_height > 0)
-			gtk_scrolled_window_set_min_content_height (scrolled_window, require_scw_height);
-		return;
-	}
-
-	if (!GTK_IS_WINDOW (toplevel) ||
-	    !gtk_widget_get_window (toplevel))
-		return;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (toplevel));
-	if (screen) {
-		gint monitor;
-		GdkRectangle workarea;
-
-		monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (toplevel));
-		if (monitor < 0)
-			monitor = 0;
-
-		gdk_screen_get_monitor_workarea (screen, monitor, &workarea);
-
-		/* can enlarge up to 4 / 5 monitor's work area height */
-		max_height = workarea.height * 4 / 5;
-	} else {
-		return;
-	}
-
-	toplevel_height = gtk_widget_get_allocated_height (toplevel);
-	if (toplevel_height + require_scw_height - scw_height > max_height)
-		return;
-
-	gtk_scrolled_window_set_min_content_height (scrolled_window, require_scw_height);
-}
-
 static void
 parts_grid_mapped_cb (GtkWidget *widget,
 		      GtkScrolledWindow *scrolled_window)
@@ -955,7 +981,7 @@ parts_grid_mapped_cb (GtkWidget *widget,
 	g_return_if_fail (GTK_IS_SCROLLED_WINDOW (scrolled_window));
 
 	ensure_scrolled_width_cb (gtk_scrolled_window_get_hadjustment (scrolled_window), NULL, scrolled_window);
-	ensure_scrolled_height_cb (gtk_scrolled_window_get_vadjustment (scrolled_window), NULL, scrolled_window);
+	e_util_ensure_scrolled_window_height (scrolled_window);
 }
 
 static GtkWidget *
@@ -1145,9 +1171,9 @@ filter_rule_get_widget (EFilterRule *rule,
 	e_signal_connect_notify (
 		hadj, "notify::upper",
 		G_CALLBACK (ensure_scrolled_width_cb), scrolledwindow);
-	e_signal_connect_notify (
+	e_signal_connect_notify_swapped (
 		vadj, "notify::upper",
-		G_CALLBACK (ensure_scrolled_height_cb), scrolledwindow);
+		G_CALLBACK (e_util_ensure_scrolled_window_height), scrolledwindow);
 
 	gtk_scrolled_window_set_policy (
 		GTK_SCROLLED_WINDOW (scrolledwindow),
