@@ -86,14 +86,14 @@ preview_pane_alert_bar_visible_notify_cb (GtkWidget *alert_bar,
 	focused = GTK_IS_WINDOW (toplevel) ? gtk_window_get_focus (GTK_WINDOW (toplevel)) : NULL;
 
 	if (!focused && preview_pane->priv->web_view &&
-	    gtk_widget_get_visible (preview_pane->priv->web_view)) {
+	    gtk_widget_is_visible (preview_pane->priv->web_view)) {
 		gtk_widget_grab_focus (preview_pane->priv->web_view);
 	}
 }
 
 static void
 preview_pane_set_web_view (EPreviewPane *preview_pane,
-                           EWebView *web_view)
+			   GtkWidget *web_view)
 {
 	gulong handler_id;
 
