@@ -98,6 +98,7 @@
 #include <e-util/e-destination-store.h>
 #include <e-util/e-dialog-utils.h>
 #include <e-util/e-dialog-widgets.h>
+#include <e-util/e-ellipsized-combo-box-text.h>
 #include <e-util/e-emoticon-action.h>
 #include <e-util/e-emoticon-chooser-menu.h>
 #include <e-util/e-emoticon-chooser.h>
@@ -113,6 +114,7 @@
 #include <e-util/e-filter-file.h>
 #include <e-util/e-filter-input.h>
 #include <e-util/e-filter-int.h>
+#include <e-util/e-filter-label.h>
 #include <e-util/e-filter-option.h>
 #include <e-util/e-filter-part.h>
 #include <e-util/e-filter-rule.h>
@@ -150,6 +152,8 @@
 #include <e-util/e-mail-signature-script-dialog.h>
 #include <e-util/e-mail-signature-tree-view.h>
 #include <e-util/e-map.h>
+#include <e-util/e-markdown-editor.h>
+#include <e-util/e-markdown-utils.h>
 #include <e-util/e-menu-tool-action.h>
 #include <e-util/e-menu-tool-button.h>
 #include <e-util/e-misc-utils.h>
