@@ -1433,7 +1433,7 @@ subscription_editor_add_store (EMSubscriptionEditor *editor,
 
 	data = g_slice_new0 (StoreData);
 	data->store = g_object_ref (store);
-	data->tree_view = g_object_ref (widget);
+	data->tree_view = GTK_TREE_VIEW (g_object_ref (widget));
 	data->list_store = GTK_TREE_MODEL (list_store);
 	data->tree_store = GTK_TREE_MODEL (tree_store);
 	data->needs_refresh = TRUE;
@@ -1544,15 +1544,8 @@ subscription_editor_dispose (GObject *object)
 
 	priv = EM_SUBSCRIPTION_EDITOR_GET_PRIVATE (object);
 
-	if (priv->session != NULL) {
-		g_object_unref (priv->session);
-		priv->session = NULL;
-	}
-
-	if (priv->initial_store != NULL) {
-		g_object_unref (priv->initial_store);
-		priv->initial_store = NULL;
-	}
+	g_clear_object (&priv->session);
+	g_clear_object (&priv->initial_store);
 
 	if (priv->timeout_id > 0) {
 		g_source_remove (priv->timeout_id);
@@ -1605,7 +1598,7 @@ subscription_editor_constructed (GObject *object)
 			e_source_get_uid (source));
 
 		if (CAMEL_IS_SUBSCRIBABLE (service))
-			editor->priv->initial_store = g_object_ref (service);
+			editor->priv->initial_store = CAMEL_STORE (g_object_ref (service));
 
 		if (service != NULL)
 			g_object_unref (service);
