@@ -276,12 +276,14 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	gboolean selection_is_instance;
 	gboolean selection_is_meeting;
 	gboolean selection_is_recurring;
+	gboolean selection_is_attendee;
 	gboolean selection_can_delegate;
 	gboolean single_event_selected;
 	gboolean refresh_supported;
 	gboolean all_sources_selected;
 	gboolean clicked_source_is_primary;
 	gboolean clicked_source_is_collection;
+	gboolean this_and_future_supported;
 
 	/* Chain up to parent's update_actions() method. */
 	E_SHELL_VIEW_CLASS (e_cal_shell_view_parent_class)->
@@ -328,10 +330,14 @@ cal_shell_view_update_actions (EShellView *shell_view)
 		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_IS_INSTANCE);
 	selection_is_meeting =
 		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_IS_MEETING);
+	selection_is_attendee =
+		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_IS_ATTENDEE);
 	selection_is_recurring =
 		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_IS_RECURRING);
 	selection_can_delegate =
 		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_CAN_DELEGATE);
+	this_and_future_supported =
+		(state & E_CAL_BASE_SHELL_CONTENT_SELECTION_THIS_AND_FUTURE_SUPPORTED);
 
 	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
 	state = e_shell_sidebar_check_state (shell_sidebar);
@@ -433,6 +439,14 @@ cal_shell_view_update_actions (EShellView *shell_view)
 		selection_is_recurring;
 	gtk_action_set_sensitive (action, sensitive);
 
+	action = ACTION (EVENT_DELETE_OCCURRENCE_THIS_AND_FUTURE);
+	sensitive =
+		single_event_selected &&
+		selection_is_editable &&
+		selection_is_recurring &&
+		this_and_future_supported;
+	gtk_action_set_sensitive (action, sensitive);
+
 	action = ACTION (EVENT_DELETE_OCCURRENCE_ALL);
 	sensitive =
 		any_events_selected &&
@@ -491,9 +505,17 @@ cal_shell_view_update_actions (EShellView *shell_view)
 	sensitive = single_event_selected && selection_is_meeting;
 	gtk_action_set_sensitive (action, sensitive);
 
-	action = ACTION (EVENT_MEETING_NEW);
+	action = ACTION (EVENT_POPUP_MEETING_NEW);
 	gtk_action_set_visible (action, has_mail_identity);
 
+	action = ACTION (EVENT_POPUP_RSVP_SUBMENU);
+	gtk_action_set_visible (action, selection_is_attendee);
+
+	sensitive = selection_is_instance || selection_is_recurring;
+	gtk_action_set_visible (ACTION (EVENT_POPUP_RSVP_ACCEPT_1), sensitive);
+	gtk_action_set_visible (ACTION (EVENT_POPUP_RSVP_DECLINE_1), sensitive);
+	gtk_action_set_visible (ACTION (EVENT_POPUP_RSVP_TENTATIVE_1), sensitive);
+
 	gtk_action_set_sensitive (ACTION (CALENDAR_GO_BACK), !is_list_view);
 	gtk_action_set_sensitive (ACTION (CALENDAR_GO_FORWARD), !is_list_view);
 	gtk_action_set_sensitive (ACTION (CALENDAR_GO_TODAY), !is_list_view);
