@@ -898,6 +898,12 @@ calendar_preferences_construct (ECalendarPreferences *prefs,
 		widget, "active",
 		G_SETTINGS_BIND_DEFAULT);
 
+	widget = e_builder_get_widget (prefs->priv->builder, "use-markdown-editor");
+	g_settings_bind (
+		settings, "use-markdown-editor",
+		widget, "active",
+		G_SETTINGS_BIND_DEFAULT);
+
 	/* These settings control the "Birthdays & Anniversaries" backend. */
 
 	eds_settings =
