@@ -25,8 +25,11 @@
 #define TABLE_ROW_URL "table_row_url"
 #define TABLE_ROW_START_DATE "table_row_start_time"
 #define TABLE_ROW_END_DATE "table_row_end_time"
+#define TABLE_ROW_DUE_DATE "table_row_due_date"
+#define TABLE_ROW_ESTIMATED_DURATION "table_row_estimated_duration"
 #define TABLE_ROW_STATUS "table_row_status"
 #define TABLE_ROW_COMMENT "table_row_comment"
+#define TABLE_ROW_CATEGORIES "table_row_categories"
 #define TABLE_ROW_DESCRIPTION "table_row_description"
 #define TABLE_ROW_RSVP_COMMENT "table_row_rsvp_comment"
 #define TABLE_ROW_ESCB "table_row_escb"
